﻿Public Class SelectorForm
  Private Const C_WINDOW_SIZE_FACTOR As Double = 0.75

  Private _ownerForm As Form
  Private _bResizing As Boolean
  Private _bResizeToFitSkipped As Boolean

  Friend Sub Open(owner As Form)
    _ownerForm = owner
    Dim tsk As New Threading.Tasks.Task(AddressOf PopulateListBox)
    tsk.Start()
    listSelect.Enabled = False
    btnSelect.Enabled = False
    listSelect.Items.Add("Loading list, please be patient...")
    Me.Size = Me.GetRecommendedFormSize(owner)
    Me.ShowDialog(owner)
  End Sub

  Private Sub PopulateListBox()
    Dim dataSource As New List(Of String)
    For i = 0 To 1000
      dataSource.Add(i.ToString)
    Next
    Threading.Thread.Sleep(2000)
    If listSelect.IsHandleCreated Then
      If listSelect.InvokeRequired Then
        listSelect.BeginInvoke(New Action(Of Object)(AddressOf PopulateListBoxUpdateUI), dataSource)
      Else
        PopulateListBoxUpdateUI(dataSource)
      End If
    End If
  End Sub

  Private Sub PopulateListBoxUpdateUI(dataSource As List(Of String))
    listSelect.Items.Clear()
    listSelect.Enabled = True
    With Me.listSelect
      .ValueMember = "Key"
      .DisplayMember = "Value"
      .DataSource = dataSource
    End With

    If _bResizing Then
      _bResizeToFitSkipped = True
      GetRecommendedFormSize(_ownerForm)
    Else
      _bResizeToFitSkipped = False
      listSelect.HorizontalScrollbar = False
      Dim recommendedFormSize As Size = GetRecommendedFormSize(_ownerForm)
      Dim screenCoord As Point = GetScreenCenterCoordinates(recommendedFormSize)
      Me.SetBounds(screenCoord.X, screenCoord.Y,
                   recommendedFormSize.Width, recommendedFormSize.Height)
      listSelect.HorizontalScrollbar = True
    End If
    pboxWaitIcon.Visible = False
    btnSelect.Enabled = True
  End Sub

  'took main part of this function from .NET CenterToScreen (using Reflector)
  Private Function GetScreenCenterCoordinates(expectedSize As Size) As Point
    Dim workingArea As Rectangle = Screen.PrimaryScreen.WorkingArea
    Dim location As New Point
    location.X = Math.Max(workingArea.X, workingArea.X + (workingArea.Width - expectedSize.Width) / 2)
    location.Y = Math.Max(workingArea.Y, workingArea.Y + (workingArea.Height - expectedSize.Height) / 2)
    Return location
  End Function

  Private Function GetMaxStringWidth() As Integer
    'calculate column width, in regard to the contents
    Dim g As System.Drawing.Graphics = listSelect.CreateGraphics
    Dim maxStringWidth As Integer = 0
    If listSelect.DataSource IsNot Nothing Then
      For Each entry As String In listSelect.DataSource
        maxStringWidth = {g.MeasureString(entry, listSelect.Font).Width, maxStringWidth}.Max
      Next
    End If
    Return maxStringWidth
  End Function

  Private Function GetRecommendedFormSize(owner As Form) As Size
    Dim maxHeight As Integer = owner.Height * C_WINDOW_SIZE_FACTOR
    Dim meHeight As Integer = maxHeight
    Dim heightDiff As Integer = Me.Height - listSelect.Height
    Dim currentListHeight As Integer = meHeight - heightDiff
    Dim maximumListHeight As Integer = (listSelect.Items.Count) * listSelect.ItemHeight + 5

    If currentListHeight > maximumListHeight Then
      meHeight -= (currentListHeight - maximumListHeight)
    End If

    Dim maxStringWidth As Integer = GetMaxStringWidth()
    listSelect.ColumnWidth = maxStringWidth

    Dim pageCount As Integer = 0
    Dim meWidth As Integer = Me.Width
    If maxStringWidth = 0 Then
      meWidth = Me.MinimumSize.Width
      meHeight = Me.MinimumSize.Height
    Else
      pageCount = Math.Ceiling(listSelect.Items.Count * listSelect.ItemHeight / currentListHeight)
      Dim widthDiff As Integer = meWidth - listSelect.Width + 5
      Dim maxWidth As Integer = owner.Width * C_WINDOW_SIZE_FACTOR
      meWidth = {pageCount, maxWidth \ maxStringWidth}.Min * maxStringWidth + widthDiff
    End If

    If listSelect.ColumnWidth < meWidth And pageCount = 1 Then listSelect.ColumnWidth = meWidth
    Return New Size(meWidth, meHeight)
  End Function

  Private Sub Me_ResizeBegin(sender As Object, e As System.EventArgs) Handles Me.ResizeBegin
    _bResizing = True
  End Sub

  Private Sub Me_ResizeEnd(sender As Object, e As System.EventArgs) Handles Me.ResizeEnd
    _bResizing = False
    If _bResizeToFitSkipped Then
      listSelect.HorizontalScrollbar = False
      Me.Size = GetRecommendedFormSize(_ownerForm)
      listSelect.HorizontalScrollbar = True
      _bResizeToFitSkipped = False
    End If
  End Sub

  Private Sub btnSelect_Click(sender As System.Object, e As System.EventArgs) Handles btnSelect.Click
    MessageBox.Show("Not implemented")
  End Sub
End Class